/*
 *  CoreAudioTest.c
 *  csound
 *
 *  Created by matt on Thu Mar 07 2002.
 *  Copyright (c) 2001 __MyCompanyName__. All rights reserved.
 *
 */

#include <CoreAudio/CoreAudio.h>
#include "cs.h"

AudioDeviceID mInputDevice;
AudioDeviceID mOutputDevice;
AudioStreamBasicDescription mInputDesc;
AudioStreamBasicDescription mOutputDesc;
AudioDeviceIOProc mInputProc = NULL;
AudioDeviceIOProc mOutputProc = NULL;

extern int csoundMain(void *csound, int argc, char **argv);
extern int runincomponents;
extern int fillbuffer(int sensEventRate);
extern int sensevents(void);
extern int cleanup(void);
extern OPARMS O;

int status = 0;
int main(int argc, char **argv)
{
    printf("Csound-CoreAudio Test");

 // call the real main, but do only the initialisation stuff
    runincomponents = 1;
    status = csoundMain(NULL, argc, argv);

    // now just loop filling buffer until done
    // obviously this is BAD
    while (status == 0)
    {
    }

    cleanup();

    return 0;
}


/*****************************************************************************
 *      CoreAudio Callbacks
 ****************************************************************************/


// our callback routine.
static OSStatus CoreAudioInputProc(       AudioDeviceID                 inDevice,
                                            const AudioTimeStamp*       inNow,
                                            const AudioBufferList*      inInputData,
                                            const AudioTimeStamp*       inInputTime,
                                            AudioBufferList*            outOutputData,
                                            const AudioTimeStamp*       inOutputTime,
                                            void*                       inClientData)
{
//      int i;

    // consume input
  //  if (inInputTime->mSampleTime)

    return kAudioHardwareNoError;

}


// our callback routine.
void *csoundOutput;
static OSStatus CoreAudioOutputProc(      AudioDeviceID                 inDevice,
                                            const AudioTimeStamp*       inNow,
                                            const AudioBufferList*      inInputData,
                                            const AudioTimeStamp*       inInputTime,
                                            AudioBufferList*            outOutputData,
                                            const AudioTimeStamp*       inOutputTime,
                                            void*                       inClientData)
{
    // produce output
    if (inOutputTime && inOutputTime->mSampleTime)
    {
        status = fillbuffer(1);
        memcpy(outOutputData->mBuffers[0].mData, csoundOutput, outOutputData->mBuffers[0].mDataByteSize);
    }

    return kAudioHardwareNoError;

}


/*****************************************************************************
 *      RealTime Audio Hooks
 ****************************************************************************/
void playopen_(int nchnls, int dsize, float sr, int scale) /* open for audio output */
{
    OSStatus status = kAudioHardwareNoError;
    AudioHardwarePropertyID propID;
    UInt32 size = 0;

    // now get info on our default output device
    size = sizeof(AudioDeviceID);
    propID = kAudioHardwarePropertyDefaultOutputDevice;
    status = AudioHardwareGetProperty(propID, &size, &mOutputDevice);
    if (status != kAudioHardwareNoError)
    {
        die("Error Getting Default Output Device\n");
    }
    size = sizeof(AudioStreamBasicDescription);
    status =  AudioDeviceGetProperty(mOutputDevice, 0, 0, kAudioDevicePropertyStreamFormat, &size, &mOutputDesc);
    if (status != kAudioHardwareNoError)
    {
        die("Error Getting Output Device Description\n");
    }

    // setcoreaudio's buffersize to the same as ours:
    // i started figuring out how to do this [commented below], but
    // got lazy... so right now you MUST set commandline to:
    // -f -b(default device's default #frames -- on my g4 it is 512)
    // with a score in stereo and sr=44100

/*        stream.mSampleRate = sr;              //      the native sample rate of the audio stream
        stream.mFormatID = kAudioFormatLinearPCM;//     the specific encoding type of audio stream
        stream.mFormatFlags = (kLinearPCMFormatFlagIsFloat & kLinearPCMFormatFlagIsBigEndian & kLinearPCMFormatFlagIsPacked);                                                                                           //      flags specific to each format
        stream.mBytesPerPacket = O.outbufsamps*dsize;// the number of bytes in a packet
        stream.mFramesPerPacket = O.outbufsamps*;       //      the number of frames in each packet
        stream.mBytesPerFrame = dsize;          //      the number of bytes in a frame
        stream.mChannelsPerFrame = nchnls;      //      the number of channels in each frame
        stream.mBitsPerChannel = dsize/nchnls;  //      the number of bits in each channel

    size = sizeof(UInt32);
    status =  AudioDeviceGetProperty(mOutputDevice, 0, 0, kAudioDevicePropertyBufferSize, &size, &bufsize);
    if (status != kAudioHardwareNoError)
    {
        die("Error Setting Output Device Buffersize\n");
    }

    status =  AudioDeviceSetProperty(mOutputDevice, 0, 0, 0, kAudioDevicePropertyBufferSize, &size, &bufsize);
  if (status != kAudioHardwareNoError)
    {
        die("Error Setting Output Device Buffersize\n");
    }

    */


    // allocate our copy-to buffer
    csoundOutput = malloc(O.outbufsamps*dsize);
    if (!csoundOutput)
        die("Error Creating Output Buffer\n");

    // startup the output device
    mOutputProc = CoreAudioOutputProc;
    status = AudioDeviceAddIOProc(mOutputDevice, mOutputProc, NULL);
    if (status != kAudioHardwareNoError)
    {
        die("Error Adding Output Device Callback\n");
    }
    status = AudioDeviceStart(mOutputDevice, mOutputProc);
     if (status != kAudioHardwareNoError)
    {
        die("Error Starting Output Device\n");
    }
}

void recopen_(int nchnls, float sr, int scale)
{
    OSStatus status = kAudioHardwareNoError;
    AudioHardwarePropertyID propID;
    UInt32 size = 0;

    // now get info on our default input device
    propID = kAudioHardwarePropertyDefaultInputDevice;
    size = sizeof(AudioDeviceID);
    status = AudioHardwareGetProperty(propID, &size, &mInputDevice);
    if (status != kAudioHardwareNoError)
    {
        die("Error Getting Default Input Device\n");
    }
    size = sizeof(AudioStreamBasicDescription);
    status =  AudioDeviceGetProperty(mInputDevice, 0, 1, kAudioDevicePropertyStreamFormat, &size, &mInputDesc);
    if (status != kAudioHardwareNoError)
    {
        die("Error Getting Input Device Description\n");
    }

        // input
    mInputProc = CoreAudioInputProc;
    status = AudioDeviceAddIOProc(mInputDevice, mInputProc, NULL);
    if (status != kAudioHardwareNoError)
    {
        die("Error Adding Input Device Callback\n");
    }
    status = AudioDeviceStart(mInputDevice, mInputProc);
    if (status != kAudioHardwareNoError)
    {
        die("Error Starting Input Device\n");
    }
}

void rtplay_(char *outbuf, int nbytes)
{
    // if we make sure -b == coreaudio's output stream buffer size
    // and ksmps < -b, then we dont have to anything here.
     // a better, more general circular-buffer approach would be buffer here

    memcpy((char *)csoundOutput, outbuf, nbytes);

}

int rtrecord_(char *inbuf, int nbytes)
{
//      if (gData.ReadAudio)
//              return (*gData.ReadAudio)(gData.owner, inbuf, nbytes);
//      else
                return(0);
}

void rtclose_(void)              /* close the I/O device entirely  */
{
    OSStatus status = kAudioHardwareNoError;

    if (mInputDevice)
    {
        status = AudioDeviceStop(mInputDevice, mInputProc);
        if (status != kAudioHardwareNoError)
        {
            die("Error Stopping Input Device\n");
        }
    }
    if (mOutputDevice)
    {
        status = AudioDeviceStop(mOutputDevice, mOutputProc);
        if (status != kAudioHardwareNoError)
        {
            die("Error Stopping Output Device\n");
        }
    }

    free(csoundOutput);
}
